#!/usr/bin/python
# -*- coding: utf-8 -*-

class Config:
    """
    Class that contains the configuration of the API.
    """
    
    HOST = 'https://www.tabnews.com.br'

    LOGIN_URL = HOST+'/api/v1/sessions'
    CONTENTS_URL = HOST+'/api/v1/contents'
    USER_DATA_URL = HOST+'/api/v1/user'
    USERS_DATA_URL = HOST+'/api/v1/users'
    
    TABNEWS_GITHUB_REPOSITORY = 'filipedeschamps/tabnews.com.br'






#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.utils import tabnews_return_validator

import requests


class PrivateRequestMixin:
    def get_headers(self):
        """
        Get the headers to be used in the requests.
        
        Returns:
        --------
            dict: The headers to be used in the requests.
        """

        return {
            "Content-Type": "application/json",
            "Cookie": f"session_id={self.session_id}",
        }

    def get(self, url, data=None):
        """
        Make a GET request to the TabNews API.
        
        Args:
        -----
            url (str): The URL to be searched for.
            data (dict): The data to be sent in the request.
            
        Returns:
        --------
            dict | object: The data of the URL.
        """
        
        headers = self.get_headers()
        return tabnews_return_validator(requests.get(url, headers=headers, json=data))

    def post(self, url, data):
        """
        Make a POST request to the TabNews API.
        
        Args:
        -----
            url (str): The URL to be searched for.
            data (dict): The data to be sent in the request.
            
        Returns:
        --------
            dict | object: The data of the URL.
        """
        
        headers = self.get_headers()
        return tabnews_return_validator(requests.post(url, headers=headers, json=data))
    
    def patch(self, url, data):
        """
        Make a PATCH request to the TabNews API.
        
        Args:
        -----
            url (str): The URL to be searched for.
            data (dict): The data to be sent in the request.
            
        Returns:
        --------
            dict | object: The data of the URL.
        """
        
        headers = self.get_headers()
        return tabnews_return_validator(requests.patch(url, headers=headers, json=data))




#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.config import Config


class CommentMixin:
    def get_comments(self, username, slug):
        """
        Get the comments of a specific content.
        
        Args:
        -----
            username (str): The username of the content's author.
            slug (str): The slug of the content.
            
        Returns:
        --------
            list: The comments of a specific content.
        """
        
        url = Config.CONTENTS_URL+'/'+username+'/'+slug+'/'+'children'
        return self.get(url)


    def publish_comment(self, parent_id, content):
        """
        Publish a comment.
        
        Args:
        -----
            parent_id (str): The id of the content to which the comment will be published.
            content (str): The content of the comment.
        
        Returns:
        --------
            dict | object: The comment's data.
        """
        
        data = {
            'parent_id': parent_id,
            'body': content,
            'status': 'published'
        }

        return self.post(Config.CONTENTS_URL, data)


    def delete_comment(self, comment_slug):
        """
        Delete a comment.
        
        Args:
        -----
            comment_slug (str): The slug of the comment.
        
        Returns:
        --------
            dict | object: The comment's data.
        """
        
        username = self.get_user()['username']
        url = Config.CONTENTS_URL+'/'+username+'/'+comment_slug
        data = {
            'status': 'deleted'
        }
        
        return self.patch(url, data)
    

    def edit_comment(self, comment_slug, parent_id, content):
        """
        Edit a comment.
        
        Args:
        -----
            comment_slug (str): The slug of the comment.
            parent_id (str): The id of the content to which the comment will be published.
            content (str): The content of the comment.
        
        Returns:
        --------
            dict | object: The comment's data.
        """
        
        username = self.get_user()['username']
        url = Config.CONTENTS_URL+'/'+username+'/'+comment_slug
        data = {
            'parent_id': parent_id,
            'body': content,
            'status': 'published'
        }
        
        self.patch(url, data)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging


class ClientError(Exception):
    def __init__(self, message):
        Exception.__init__(self)
        self.message = message
        self.logger = logging.getLogger('TabNews')
        self.logger.error(f'\n{self.message}\n')

class LoginRequired(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)
    
class InvalidCredentials(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)

class InvalidTabnewsReturn(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)

class BadTabnewsRequest(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)

class BadUrl(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)

class InsufficientTabcoins(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)

class PreviewHostError(ClientError):
    def __init__(self, message):
        ClientError.__init__(self, message)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.utils import url_validator
from tabnews.exceptions import BadUrl
from tabnews.config import Config


class PostMixin:
    def get_post(self, username, slug):
        """
        Get a specific content.
        
        Args:
        -----
            username (str): The username of the content's author.
            slug (str): The slug of the content.
            
        Returns:
        --------
            dict | object: The content's data.
        """
        
        url = Config.CONTENTS_URL+'/'+username+'/'+slug
        return self.get(url)


    def list_posts_from_url(self, url, all_posts=False):
        """
        List the contents of a specific URL.
        
        Args:
        -----
            url (str): The URL to be searched for.
            all_posts (bool): If True, all the contents of the URL will be returned.
            
        Returns:
        --------
            list: The contents of a specific URL.
        """
        
        posts = []
        page = 1
        while True:
            if '?' in url:
                prefix = '&'
            else:
                prefix = '?'

            data = self.get(url+f'{prefix}page={page}')
            if not data:
                break
            posts.extend(data)
            if not all_posts:
                break
            page += 1
        return posts


    def get_posts_from_user(self, username, all_posts=False):
        """
        Get the contents of a specific user.
        
        Args:
        -----
            url (str): The URL to be searched for.
            all_posts (bool): If True, all the contents of the URL will be returned.
            
        Returns:
        --------
            list: The contents of a specific URL.
        """
        
        url = Config.CONTENTS_URL+'/'+username
        return self.list_posts_from_url(url, all_posts)
    

    def get_relevant_posts(self, all_posts=False):
        """
        Get the relevant contents.
        
        Args:
        -----
            url (str): The URL to be searched for.
            all_posts (bool): If True, all the contents of the URL will be returned.
            
        Returns:
        --------
            list: The contents of a specific URL.
        """
        
        url = Config.CONTENTS_URL+'?strategy=relevant'
        return self.list_posts_from_url(url, all_posts)	
    

    def get_new_posts(self, all_posts=False):
        """
        Get the new contents.
        
        Args:
        -----
            url (str): The URL to be searched for.
            all_posts (bool): If True, all the contents of the URL will be returned.
            
        Returns:
        --------
            list: The contents of a specific URL.
        """
        
        url = Config.CONTENTS_URL+'?strategy=new'
        return self.list_posts_from_url(url, all_posts)
    

    def get_old_posts(self, all_posts=False):
        """
        Get the old contents.
        
        Args:
        -----
            url (str): The URL to be searched for.
            all_posts (bool): If True, all the contents of the URL will be returned.
            
        Returns:
        --------
            list: The contents of a specific URL.
        """
        
        url = Config.CONTENTS_URL+'?strategy=old'
        return self.list_posts_from_url(url, all_posts)


    def publish_post(self, title, content, reference=None):
        """
        Publish a content.
        
        Args:
        -----
            title (str): The title of the content.
            content (str): The content.
            reference (str): The URL of the content's source.
            
        Returns:
        --------
            dict | object: The content's data.
        """
        
        data = {
            "title": title,
            'body': content,
            'status': 'published'
        }

        if reference:
            if not url_validator(reference):
                raise BadUrl('A URL de referência fornecida para a postagem não é válida.')
            data['source_url'] = reference
            
        return self.post(Config.CONTENTS_URL, data)


    def delete_post(self, slug):
        """
        Delete a content.
        
        Args:
        -----
            slug (str): The slug of the content.
            
        Returns:
        --------
            dict | object: The content's data.
        """
        
        username = self.get_user().username
        url = Config.CONTENTS_URL+'/'+username+'/'+slug
        data = {
            'status': 'deleted'
        }
        
        return self.patch(url, data)


    def edit_post(self, username, slug, title, content, reference=None):
        """
        Edit a content.
        
        Args:
        -----
            username (str): The username of the content's author.
            slug (str): The slug of the content.
            title (str): The title of the content.
            content (str): The content.
            reference (str): The URL of the content's source.
        
        Returns:
        --------
            dict | object: The content's data.
        """
        
        url = Config.CONTENTS_URL+'/'+username+'/'+slug
        data = {
            "title": title,
            'body': content,
            'status': 'published'
        }

        if reference:
            if not url_validator(reference):
                raise BadUrl('A URL de referência fornecida para a edição da postagem não é válida.')
            data['source_url'] = reference
            
        return self.patch(url, data)





#!/usr/bin/python
# -*- coding: utf-8 -*-

import requests, json, os

from tabnews.exceptions import InvalidCredentials, LoginRequired
from tabnews.config import Config


class LoginMixin:
    def login(self):
        """
        Login to the TabNews API.
        
        Returns:
        --------
            dict | object: The user's data.
        """
        
        if self.email is None or self.password is None:
            raise LoginRequired("É necessário informar o email e a senha do TabNews")

        url = Config.LOGIN_URL
        data = {
            "email": self.email,
            "password": self.password,
        }

        headers = {
            "Content-Type": "application/json",
        }

        response = requests.post(url, json=data, headers=headers)
        if response.status_code == 200 or response.status_code == 201:
            self.session_id = response.json()["token"]
            self.logger.info("Logged in")
            return self.get_user()

        else:
            try:
                raise InvalidCredentials(f'{response.json()["message"]}\n{response.json()["action"]}\nStatus code: {response.status_code}')
            except KeyError:
                raise InvalidCredentials("Credenciais inválidas ou ocorreu um erro no endpoint do TabNews")


    def load_config(self, use_preview_tabnews_host=False, path='config.json'):
        """
        Load the user's session id and configuration from a file.
        
        Args:
        -----
            use_preview_tabnews_host (bool): If True, the session id will be loaded from the preview tabnews host.
            path (str): The path to the file.
        
        Returns:
        --------
            str: The user's session id.
        """
        
        try:
            if self.config_path != path:
                self.config_path = path

            with open(path, 'r') as f:
                config = json.load(f)
                for session in config:
                    host = 'production'
                    if use_preview_tabnews_host == True:
                        host = 'preview'
                    if session['email'] == self.email and session['host'] == host:
                        self.session_id = session['session_id']
                        return self.session_id

        except json.decoder.JSONDecodeError:
            raise json.decoder.JSONDecodeError("Arquivo de configuração inválido, verifique se o arquivo está no formato JSON")

        except FileNotFoundError:
            raise FileNotFoundError("Arquivo de configuração não encontrado")

        except KeyError:
            raise KeyError("Arquivo de configuração inválido")


    def dump_config(self, use_preview_tabnews_host=False, path='config.json'):
        """
        Save the user's session id and configuration to a file.
        
        Args:
        -----
            use_preview_tabnews_host (bool): If True, the session id will be saved to the preview tabnews host.
            path (str): The path to the file.
        """
        
        if self.config_path != path:
            self.config_path = path

        user_not_saved = True
        if not os.path.exists(path):
            with open(path, 'w') as f:
                pass
            
        with open(path, 'r+') as f:
            host = 'production'
            if use_preview_tabnews_host == True:
                host = 'preview'
                
            data = f.read()
            if data == '':
                list_sessions = []
            else:
                list_sessions = json.loads(data)
            
            for session in list_sessions:
                if session['email'] == self.email and session['host'] == host:
                    user_not_saved = False
                    session['session_id'] = session

            if user_not_saved:
                list_sessions.append({
                    'session_id': self.session_id,
                    'email': self.email,
                    'host': host,
                })

            f.seek(0)
            
            json.dump(list_sessions, f, indent=4)
            f.truncate()




#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.config import Config


class GetUserMixin:
    def get_user(self, username=None):
        """
        Get the user's data or the data of a specific user.
        
        Args:
        -----
            username (str): The username of the user to be searched for.
            
        Returns:
        --------
            dict | object: The user's data or the data of a specific user.
        """
        
        if username is None:
            url = Config.USER_DATA_URL
            response = self.get(url)
            return response
        
        else:
            url = Config.USERS_DATA_URL+f'/{username}'
            response = self.get(url)
            response = response

            # These additional fields serve to match the fields of searching 
            # for a user with that of searching for the user himself, since 
            # they have different returns for security reasons
            response['email'] = None
            response['notifications'] = None

            return response





#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.mixins.auth import LoginMixin
from tabnews.mixins.private import PrivateRequestMixin
from tabnews.mixins.account import GetUserMixin
from tabnews.mixins.posts import PostMixin
from tabnews.mixins.comments import CommentMixin
from tabnews.mixins.tabcoins import TabcoinsMixin

from tabnews.utils import get_preview_url
from tabnews.config import Config

import logging, os


DEFAULT_LOGGER = logging.getLogger('TabNews')


class Client(
    LoginMixin,
    PrivateRequestMixin,
    GetUserMixin,
    PostMixin,
    TabcoinsMixin,
    CommentMixin,
):
    """
    The TabNews client. 
    """
    
    def __init__(self, email=None, password=None, token=None, save_session=True, config_path='config.json', use_preview_tabnews_host=False, logger=DEFAULT_LOGGER):
        self.email = email or os.environ.get('TABNEWS_EMAIL')
        self.password = password or os.environ.get('TABNEWS_PASSWORD')
        self.logger = logger
        self.config_path = config_path
        if use_preview_tabnews_host == True:
            new_host = get_preview_url()
            atual_host = Config.HOST
            self.logger.info(f"Usando o ambiente de homologação: {new_host}")
            
            for key, value in Config.__dict__.items():
                if isinstance(value, str):
                    setattr(Config, key, value.replace(atual_host, new_host))

        if token is None:
            if save_session == True:
                if os.path.exists(self.config_path):
                    self.session_id = self.load_config(use_preview_tabnews_host)
                    if self.session_id is None:
                        self.login()
                        self.dump_config(use_preview_tabnews_host)
                        self.logger.info("Session loaded")

                else:
                    self.login()
                    self.dump_config(use_preview_tabnews_host)
                    self.logger.info("Session saved")

            else:
                self.login()
        
        else:
            self.session_id = token



#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.exceptions import InsufficientTabcoins
from tabnews.config import Config


class TabcoinsMixin:
    def upvote(self, username, slug_or_parent_id, upvote_amount=1):
        """
        Upvote a content.
        
        Args:
        -----
            username (str): The username of the content's author.
            slug_or_parent_id (str): The slug or parent_id of the content.
            upvote_amount (int): The amount of upvotes to be given.
        
        Returns:
        --------
            dict | object: The content's data.
        """
        
        tabcoins_limit = self.get_user()['tabcoins']
        url = Config.CONTENTS_URL+'/'+username+'/'+slug_or_parent_id+'/tabcoins'
        data = {
            "transaction_type": "credit"
        }

        if upvote_amount*2 > tabcoins_limit:
            raise InsufficientTabcoins(f'Você não possui tabcoins suficientes para realizar essa ação!\nVocê possui {tabcoins_limit} tabcoins, mas precisa de {upvote_amount*2} para realizar essa ação.')
        else:
            result = {'tabcoins': None}
            for i in range(upvote_amount):
                result = self.post(url, data)

            return result

    def downvote(self, username, slug_or_parent_id, downvote_amount=1):
        """
        Downvote a content.
        
        Args:
        -----
            username (str): The username of the content's author.
            slug_or_parent_id (str): The slug or parent_id of the content.
            upvote_amount (int): The amount of upvotes to be given.
        
        Returns:
        --------
            dict | object: The content's data.
        """
        
        tabcoins_limit = self.get_user()['tabcoins']
        url = Config.CONTENTS_URL+'/'+username+'/'+slug_or_parent_id+'/tabcoins'
        data = {
            "transaction_type": "debit"
        }

        if downvote_amount*2 > tabcoins_limit:
            raise InsufficientTabcoins(f'Você não possui tabcoins suficientes para realizar essa ação!\nVocê possui {tabcoins_limit} tabcoins, mas precisa de {downvote_amount*2} para realizar essa ação.')
        else:
            result = {'tabcoins': None}
            for i in range(downvote_amount):
                result = self.post(url, data)

            return result





#!/usr/bin/python
# -*- coding: utf-8 -*-

import setuptools


with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()
    
with open("requirements.txt", "r", encoding="UTF-8") as fh:
    requirements = fh.read().splitlines()

setuptools.setup(
    name="tabnews",
    version="1.0.0",
    author="Gustavo Santana",
    license='MIT',
    author_email="sowlfie@gmail.com",
    description='Python library for communication via API to the website: "TabNews.com.br"',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    keywords=["api", "tabnews", "tabcoins"],
    url="https://github.com/Gustavosta/Tabnews.py",
    packages=setuptools.find_packages(),
    python_requires=">=3.7",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




#!/usr/bin/python
# -*- coding: utf-8 -*-

from tabnews.exceptions import InvalidTabnewsReturn, BadTabnewsRequest, PreviewHostError
from tabnews.config import Config

from cleverdict import CleverDict
import json, re, requests


def get_preview_url():
    """
    Get the preview URL on GitHub repository of the Tabnews.
    
    Returns:
    --------
        str: The preview URL of the Tabnews.
    """
    
    try:
        url = F'https://api.github.com/repos/{Config.TABNEWS_GITHUB_REPOSITORY}/deployments'
        response = requests.get(url).json()
        id = None
        
        for deployment in response:
            if deployment['environment'].lower() == 'preview':
                id = deployment['id']
                break
            
        if id is None:
            raise ValueError('No deployment found')
        
        else:
            url = f'https://api.github.com/repos/{Config.TABNEWS_GITHUB_REPOSITORY}/deployments/{id}/statuses'
            response = requests.get(url).json()
            
            for status in response:
                if status['state'] == 'success':
                    return status['target_url']
                
    except:
        raise PreviewHostError('Não foi possível obter o host do homologação do Tabnews.')


def url_validator(url):
    """
    Validate the URL.
    
    Args:
    -----
        url (str): The URL to be validated.
        
    Returns:
    --------
        bool: True if the URL is valid, False otherwise.
    """
    
    regex = "^https?:\\/\\/(?:www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{1,256}\\.[a-zA-Z0-9()]{1,6}\\b(?:[-a-zA-Z0-9()@:%_\\+.~#?&\\/=]*)$"
    return True if re.search(regex, url) else False


def dict_and_object(dictonary):
    """
    Convert a dict to a CleverDict object.
    """
    
    if type(dictonary) == dict:
        return CleverDict(dictonary)
    return dictonary


def tabnews_return_validator(response):
    """
    Validate the return of the Tabnews API.
    
    Args:
    -----
        response (str | dict): The response of the Tabnews API.
        
    Returns:
    --------
        dict | object: The response of the Tabnews API.
    """
    
    try:
        if type(response) != str and type(response) != dict:
            response = response.text

        json_loaded = json.loads(response)

        if 'action' in json_loaded:
            raise BadTabnewsRequest(f'Bad request: {json_loaded["message"]}\n{json_loaded["action"]}')
        return dict_and_object(json_loaded)

    except ValueError:
        raise InvalidTabnewsReturn('O retorno da requisição não é um JSON válido, verifique o status de funcionamento do TabNews.')



